@@ -1,3 +1,5 @@
+#include "config.h"
+
 #ifdef HAVE_MACHINE_SOUNDCARD_H
 #  include <machine/soundcard.h>
 #else
@@ -70,13 +72,14 @@ int esd_audio_open()
 #endif /* #if !defined(__powerpc__) */
 
     /* set the sound driver audio format for playback */
-#if defined(__powerpc__) || defined(__sparc__)
+    
+#if defined(WORDS_BIGENDIAN)
     value = test = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
-        ? /* 16 bit */ AFMT_S16_NE : /* 8 bit */ AFMT_U8;
-#else /* #if !defined(__powerpc__) */
+        ? /* 16 bit */ AFMT_S16_BE : /* 8 bit */ AFMT_U8;
+#else
     value = test = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
         ? /* 16 bit */ AFMT_S16_LE : /* 8 bit */ AFMT_U8;
-#endif /* #if !defined(__powerpc__) */
+#endif
 
     if (ioctl(afd, SNDCTL_DSP_SETFMT, &test) == -1)
     {   /* Fatal error */
