@@ -21,8 +21,16 @@ int deny_severity = LOG_WARNING;
 /* the list of the currently connected clients */
 esd_client_t *esd_clients_list;
 
+/*******************************************************************/
+/* prototypes */
+void dump_clients(void);
+void free_client( esd_client_t *client );
+
+
 /*******************************************************************/
 /* for debugging purposes, dump the list of the clients and data */
+
+
 void dump_clients()
 {
     long addr;
@@ -35,7 +43,7 @@ void dump_clients()
 	port = ntohs( clients->source.sin_port );
 	addr = ntohl( clients->source.sin_addr.s_addr );
 
-	printf( "(%02d) client from: %03u.%03u.%03u.%03u:%05d [%08p]\n", 
+	printf( "(%02d) client from: %03u.%03u.%03u.%03u:%05d [%p]\n", 
 		clients->fd, (unsigned int) addr >> 24, 
 		(unsigned int) (addr >> 16) % 256, 
 		(unsigned int) (addr >> 8) % 256, 
@@ -150,7 +158,6 @@ int get_new_clients( int listen )
 	    }
 #endif
 
-	    /* if ( esdbg_comms ) printf( "================================\n" ); */
 	    ESDBG_COMMS( printf( "================================\n" ); );
 
 	    /* make sure we have the memory to save the client... */
@@ -173,33 +180,16 @@ int get_new_clients( int listen )
 		return -1;
 	    }
 
-	  /* Reduce buffers on sockets to the minimum needed */
-	    {
-	      int fd_bufsize;
-	      int src_format;
-	      int src_rate;
-	      extern int esd_audio_rate;
+	    /* Reduce buffers on sockets to the minimum needed */
+	    esd_set_socket_buffers( fd, ESD_BITS16, 44100, esd_audio_rate );
 
-	      src_rate = 44100;
-	      src_format = ESD_BITS16;
-	      fd_bufsize = ESD_BUF_SIZE;
-	      fd_bufsize = (fd_bufsize * esd_audio_rate) / src_rate;
-	      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-		fd_bufsize *= 2;
-	      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-		fd_bufsize *= 2;	      
-	      setsockopt(fd, SOL_SOCKET, SO_SNDBUF, 
-			 &fd_bufsize, sizeof(fd_bufsize));
-	      setsockopt(fd, SOL_SOCKET, SO_RCVBUF,  
-			 &fd_bufsize, sizeof(fd_bufsize));
-	    }
 	    /* fill in the new_client structure - sockaddr = works!? */
-	    /* request = ..._INVALID forces polling client next time */
 	    new_client->next = NULL;
-	    new_client->state = ESD_NEEDS_VALIDATION;
-	    new_client->request = ESD_PROTO_INVALID;
+	    new_client->state = ESD_NEEDS_REQDATA;
+	    new_client->request = ESD_PROTO_CONNECT;
 	    new_client->fd = fd;
 	    new_client->source = incoming; 
+	    new_client->proto_data_length = 0;
 	    
 	    add_new_client( new_client );
 	}
@@ -208,6 +198,7 @@ int get_new_clients( int listen )
     return 0;
 }
 
+static int is_paused_here = 0;
 /*******************************************************************/
 /* blocks waiting for data from the listener, and client conns. */
 int wait_for_clients_and_data( int listen )
@@ -216,7 +207,7 @@ int wait_for_clients_and_data( int listen )
     struct timeval timeout;
     struct timeval *timeout_ptr = NULL;
     esd_client_t *client = esd_clients_list;
-    int max_fd = listen;
+    int max_fd = listen, ready;
 
     /* add the listener to the file descriptor list */
     FD_ZERO( &rd_fds );
@@ -225,7 +216,14 @@ int wait_for_clients_and_data( int listen )
     /* add the clients to the list, too */
     while ( client != NULL )
     {
-	/* add this client */
+	/* add this client, but only if it's not monitoring */
+	if ( client->state == ESD_STREAMING_DATA &&
+	     client->request == ESD_PROTO_STREAM_MON )
+	{
+	    client = client->next;
+	    continue;
+	}
+
 	FD_SET( client->fd, &rd_fds );
 
 	/* update the maximum fd for the select() */
@@ -237,24 +235,76 @@ int wait_for_clients_and_data( int listen )
     }
 
     /* if we're doing something useful, make sure we return immediately */
-    if ( esd_recorder ) {
+    if ( esd_recorder || esd_playing_samples ) {
 	timeout.tv_sec = 0;
 	timeout.tv_usec = 0;
 	timeout_ptr = &timeout;
     } else {
 
-	timeout.tv_sec = 0;
-	/* funky math to make sure a long can hold it all, calulate in ms */
-	timeout.tv_usec = (long) esd_buf_size_samples * 1000L
-	    / (long) esd_audio_rate / 4L; 	/* divide by two for stereo */
-	timeout.tv_usec *= 1000L; 		/* convert to microseconds */
-	timeout_ptr = &timeout;
+	/* TODO: any reason not to pause indefinitely here? */
+	/* sample players that's why, if no players, can pause indefinitely */
+	/* if ( esd_on_autostandby 
+	        || (esd_autostandby_secs < 0 && !esd_playing_samples ) )
+	       timeout_ptr = NULL; else { ... } */
 
-	/* we might not be doing something useful, kill audio echos */
-	esd_audio_pause();
+	if ( is_paused_here ) {
+
+	    ESDBG_TRACE( printf( "paused, awaiting instructions.\n" ); );
+	    timeout_ptr = NULL;
+
+	} else {
+
+	    timeout.tv_sec = 0;
+	    /* funky math to make sure a long can hold it all, calulate in ms */
+	    timeout.tv_usec = (long) esd_buf_size_samples * 1000L
+		/ (long) esd_audio_rate / 4L; 	/* divide by two for stereo */
+	    timeout.tv_usec *= 1000L; 		/* convert to microseconds */
+	    timeout_ptr = &timeout;
+
+	}
     }
 
-    select( max_fd+1, &rd_fds, NULL, NULL, timeout_ptr );
+    ready = select( max_fd+1, &rd_fds, NULL, NULL, timeout_ptr );
 
-    return 0;
+    ESDBG_COMMS( printf( 
+	"paused=%d, samples=%d, auto=%d, standby=%d, record=%d, ready=%d\n",
+	is_paused_here, esd_playing_samples, 
+	esd_autostandby_secs, esd_on_standby, 
+	(esd_recorder != 0), ready ); );
+
+    /* TODO: return ready, and do this in esd.c */
+    if ( ready <= 0 ) {
+	/* if < 0, something horrible happened:
+	   EBADF   invalid file descriptor - let individual read sort it out
+	   EINTR   non blocked signal caught - o well, no big deal
+	   EINVAL  n is negative - not bloody likely
+	   ENOMEM  unable to allocate internal tables - o well, no big deal */
+
+	if ( !is_paused_here && !esd_playing_samples && (esd_autostandby_secs<0) ) {
+	    ESDBG_TRACE( printf( "doing nothing, pausing server.\n" ); );
+	    esd_audio_flush();
+	    esd_audio_pause();
+	    esd_last_activity = time( NULL );
+	    is_paused_here = 1;
+	}
+
+	if ( !is_paused_here && !esd_playing_samples && !esd_recorder ) {
+
+	    if ( esd_autostandby_secs >= 0
+		 && ( time(NULL) > esd_last_activity + esd_autostandby_secs ) ) {
+		ESDBG_TRACE( printf( "bored, going to standby mode.\n" ); );
+		esd_server_standby();
+		esd_on_autostandby = 1;
+		is_paused_here = 1;
+	    }
+
+	}
+
+    } else {
+
+	is_paused_here = 0;
+
+    }
+
+    return ready;
 }
