@@ -33,6 +33,7 @@ int esd_proto_all_info( esd_client_t *client );
 int esd_proto_stream_pan( esd_client_t *client );
 int esd_proto_sample_pan( esd_client_t *client );
 int esd_proto_standby_mode( esd_client_t *client );
+int esd_proto_get_latency( esd_client_t *client );
 int poll_client_requests(void);
 
 /*******************************************************************/
@@ -68,7 +69,7 @@ esd_proto_handler_info_t esd_proto_map[ ESD_PROTO_MAX ] =
     { ESD_KEY_LEN + sizeof(int), &esd_proto_resume, "resume" },
 
     { ESD_NAME_MAX, &esd_proto_sample_getid, "sample getid" },
-    { ESD_KEY_LEN + 2 * sizeof(int), &esd_proto_stream_filter, "stream filter" },
+    { ESD_NAME_MAX + 2 * sizeof(int), &esd_proto_stream_filter, "stream filter" },
 
     { sizeof(int), &esd_proto_server_info, "server info" },
     { sizeof(int), &esd_proto_all_info, "all info" },
@@ -78,9 +79,42 @@ esd_proto_handler_info_t esd_proto_map[ ESD_PROTO_MAX ] =
     { 3 * sizeof(int), &esd_proto_stream_pan, "stream pan"},
     { 3 * sizeof(int), &esd_proto_sample_pan, "sample pan" },
 
-    { sizeof(int), &esd_proto_standby_mode, "standby mode" }
+    { sizeof(int), &esd_proto_standby_mode, "standby mode" },
+    { 0, &esd_proto_get_latency, "get latency" }
 };
 
+/***********************************************************************/
+/* returns the latency between audio stream data being write() 'en to  */
+/* esd and when it finally comes out the speakers of your audio system */
+int esd_proto_get_latency( esd_client_t *client )
+{
+    int lag, amount, actual;
+
+    ESDBG_TRACE( printf( "(%02d) proto: get latency\n", client->fd ); );
+  
+    if (esd_audio_format & ESD_STEREO)
+      {
+	if (esd_audio_format & ESD_BITS16)
+	  amount = (44100 * (ESD_BUF_SIZE + 64)) / esd_audio_rate;
+	else
+	  amount = (44100 * (ESD_BUF_SIZE + 128)) / esd_audio_rate;
+      }
+    else
+      {
+	if (esd_audio_format & ESD_BITS16)
+	  amount = (2 * 44100 * (ESD_BUF_SIZE + 128)) / esd_audio_rate;
+	else
+	  amount = (2 * 44100 * (ESD_BUF_SIZE + 256)) / esd_audio_rate;
+      }
+    lag = maybe_swap_32( client->swap_byte_order, amount );
+
+    /* send back the server information */
+    ESD_WRITE_INT( client->fd, &lag, sizeof(lag), actual, "lag buf" );
+    if ( sizeof( lag ) != actual )
+	return 0;
+    return 1;
+}
+
 /*******************************************************************/
 /* checks for client/server endianness */
 int esd_check_endian( esd_client_t *client, unsigned int *endian )
@@ -110,6 +144,7 @@ int esd_validate_source( esd_client_t *client,
 			 octet *submitted_key,
 			 int owner_only )
 {
+  int ok;
     if( !esd_is_owned ) {
 	/* noone owns it yet, the first client claims ownership */
 	ESDBG_TRACE( printf( "(%02d) esd auth: claiming ownership of esd, auth ok\n", 
@@ -118,6 +153,8 @@ int esd_validate_source( esd_client_t *client,
 	esd_is_locked = 1;
 	memcpy( esd_owner_key, submitted_key, ESD_KEY_LEN );
 	esd_is_owned = 1;
+        ok = 1;
+        write(client->fd, &ok, sizeof(ok));
 	return 1;
     }
 
@@ -125,6 +162,8 @@ int esd_validate_source( esd_client_t *client,
 	/* anyone can connect to it */
 	ESDBG_TRACE( printf( "(%02d) esd auth: not locked nor owner only, auth ok.\n", 
 			     client->fd ); );
+        ok = 1;
+        write(client->fd, &ok, sizeof(ok));
 	return 1;
     }
 
@@ -132,6 +171,8 @@ int esd_validate_source( esd_client_t *client,
 	/* the client key matches the owner, trivial acceptance */
 	ESDBG_TRACE( printf( "(%02d) esd auth: key matches, auth ok.\n", 
 			     client->fd ); );
+        ok = 1;
+        write(client->fd, &ok, sizeof(ok));
 	return 1;
     }
 
@@ -142,6 +183,8 @@ int esd_validate_source( esd_client_t *client,
     /* the client is not authorized to connect to the server */
     ESDBG_TRACE( printf( "(%02d) esd auth: NOT authorized to use esd, closing conn.\n",
 			 client->fd ); );
+    ok = 0;
+    write(client->fd, &ok, sizeof(ok));
     return 0;
 }
 
@@ -163,7 +206,7 @@ int esd_proto_connect( esd_client_t *client )
 
     if ( esd_validate_source( client, client->proto_data, 0 ) )
     {
-	ok = esd_check_endian( client, (int*)(client->proto_data + ESD_KEY_LEN) );
+	ok = esd_check_endian( client, (unsigned int*)(client->proto_data + ESD_KEY_LEN) );
     } else {
 	ok = 0;
     }
@@ -899,6 +942,15 @@ int poll_client_requests()
 		/* TODO: do one read, and handle if we get all the data */
 		/* this should fix the "all handlers requrie data oddity */
 		is_ok = 1;
+	        /* if this request requires no additional param data... */
+	        if (esd_proto_map[client->request].data_length == 0)
+		  {
+		    ESDBG_TRACE( printf( "(%02d) handling request %d, %s.\n", 
+					client->fd, client->request, 
+					esd_proto_map[ client->request ].description ); );
+		    client->state = ESD_NEXT_REQUEST; /* handler may override */
+		    is_ok = esd_proto_map[ client->request ].handler( client );
+		  }
 	    } 
 	    else {
 		ESDBG_TRACE( printf( "(%02d) invalid request: %d\n",
